#include "mainview.h"

#include <QtAndroidExtras/QAndroidJniObject>
#include <QtAndroidExtras/QAndroidJniEnvironment>

#include <QDebug>
#include <QQuickItem>

void MainView::errorCallback(JNIEnv *env, jobject thiz, jlong handle, jstring errmsg)
{
    Q_UNUSED(thiz);
    MainView *nw = reinterpret_cast<MainView*>(handle);
    const jchar* c = env->GetStringChars(errmsg, nullptr);
    QString msg = QString::fromUtf16(c);
    emit nw->error(msg);
}

void MainView::completeCallback(JNIEnv *env, jobject thiz, jlong handle, jbyteArray res)
{
    Q_UNUSED(thiz);
    MainView *nw = reinterpret_cast<MainView*>(handle);
    if(res){
        jboolean cpy;
        jsize sz = env->GetArrayLength(res);
        jbyte *buf = env->GetByteArrayElements(res, &cpy);
        QByteArray *ba = new QByteArray(reinterpret_cast<char*>(buf), sz);
        env->ReleaseByteArrayElements(res, buf, 0);
        delete ba;
    }

    emit nw->complete();
}

MainView::MainView()
{
    this->setResizeMode(QQuickView::SizeRootObjectToView);
    this->setSource(QUrl("qrc:/main.qml"));

    connect(this->rootObject(), SIGNAL(callJni()), this, SLOT(onCallJni()));


    QAndroidJniEnvironment env;

    JniTestClass = new QAndroidJniObject(JniTestClassName,
        "(J)V", reinterpret_cast<jlong>(this));

    testClass = env->GetObjectClass(JniTestClass->object());
    _begin = env->GetMethodID(testClass, "begin", "()V");
    _download = env->GetMethodID(testClass, "download", "(Ljava/lang/String;)V");

    JNINativeMethod pmethods[] {
        {"error", "(JLjava/lang/String;)V", reinterpret_cast<void *>(MainView::errorCallback)},
        {"complete", "(J[B)V", reinterpret_cast<void *>(MainView::completeCallback)},
    };
    env->RegisterNatives(testClass, pmethods, 2);

    jboolean res = QAndroidJniObject::callStaticMethod<jboolean>(JniTestClassName, "initialize", "(I)Z", 4);
}

void MainView::onCallJni()
{
    QAndroidJniEnvironment env;

    env->CallVoidMethod(JniTestClass->object(), _begin);

    QAndroidJniObject jmsg = QAndroidJniObject::fromString("Hello from Qt C++.");
    env->CallVoidMethod(JniTestClass->object(), _download, jmsg.object<jstring>());

}
